@@ -5,9 +5,13 @@
     copyright            : (C) 2002 by ARRL
     author               : Jon Bloom
     email                : jbloom@arrl.org
-    revision             : $Id: cabrillo.cpp,v 1.5 2005/02/18 20:21:01 ke3z Exp $
+    revision             : $Id: cabrillo.cpp,v 1.7 2010/03/09 16:46:49 k1mu Exp $
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include "sysconfig.h"
+#endif
+
 #define TQSLLIB_DEF
 
 #include "tqsllib.h"
@@ -227,7 +231,7 @@ freq_to_band(TQSL_CABRILLO *cab, tqsl_cabrilloField *fp) {
 		return 0;
 	}
 	int freq = atoi(fp->value);
-	char *band = 0;
+	const char *band = 0;
 	if (cab->contest->type == TQSL_CABRILLO_HF) {
 		if (freq < 30) {
 			// Handle known CT misbehavior
@@ -303,8 +307,8 @@ freq_to_band(TQSL_CABRILLO *cab, tqsl_cabrilloField *fp) {
 static int
 mode_xlat(TQSL_CABRILLO *cab, tqsl_cabrilloField *fp) {
 	static struct {
-		char *cmode;
-		char *gmode;
+		const char *cmode;
+		const char *gmode;
 	} modes[] = {
 		{"CW", "CW"}, {"PH", "Phone"}, {"FM", "Phone"}, {"RY", "RTTY"}
 	};
@@ -472,7 +476,7 @@ tqsl_endCabrillo(tQSL_Cabrillo *cabp) {
 
 DLLEXPORT const char *
 tqsl_cabrilloGetError(TQSL_CABRILLO_ERROR_TYPE err) {
-	char *msg = 0;
+	const char *msg = 0;
 	switch (err) {
 		case TQSL_CABRILLO_NO_ERROR:
 			msg = "Cabrillo success";
