@@ -5,7 +5,7 @@
     copyright            : (C) 2002 by ARRL
     author               : Jon Bloom
     email                : jbloom@arrl.org
-    revision             : $Id: tqslconvert.cpp,v 1.7 2005/02/18 20:21:01 ke3z Exp $
+    revision             : $Id: tqslconvert.cpp,v 1.8 2010/03/09 01:09:23 k1mu Exp $
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -24,6 +24,7 @@
 #include <ctype.h>
 #include <set>
 
+#include <locale.h>
 //#include <iostream>
 
 using namespace std;
@@ -264,6 +265,7 @@ static const char *notypes[] = { "D","T","M","N","C","" };
 
 static const char *
 tqsl_infer_band(const char* infreq) {
+	setlocale(LC_NUMERIC, "C");
 	double freq = atof(infreq);
 	double freq_khz = freq * 1000.0;
 	int nband = 0;
