@@ -5,7 +5,7 @@
     copyright            : (C) 2002 by ARRL
     author               : Jon Bloom
     email                : jbloom@arrl.org
-    revision             : $Id: tqsllib.h,v 1.9 2005/02/23 18:09:25 ke3z Exp $
+    revision             : $Id: tqsllib.h,v 1.12 2010/09/28 17:43:52 ke3z Exp $
  ***************************************************************************/
 
 #ifndef TQSLLIB_H
@@ -343,6 +343,16 @@ DLLEXPORT int tqsl_getCertificateEncoded(tQSL_Cert cert, char *buf, int bufsiz);
   */
 DLLEXPORT int tqsl_getCertificateSerial(tQSL_Cert cert, long *serial);
 
+/** Get the issuer's serial number of the certificate as a hexadecimal string.
+  * Needed for certs with long serial numbers (typically root certs).
+  */
+DLLEXPORT int tqsl_getCertificateSerialExt(tQSL_Cert cert, char *serial, int serialsiz);
+
+/** Get the length of the issuer's serial number of the certificate as it will be
+  * returned by tqsl_getCertificateSerialExt.
+  */
+DLLEXPORT int tqsl_getCertificateSerialLength(tQSL_Cert cert);
+
 /** Get the issuer (DN) string from a tQSL_Cert.
   *
   * \li \c cert - a tQSL_Cert object, normally one returned from
@@ -576,6 +586,16 @@ DLLEXPORT void tqsl_freeCertificate(tQSL_Cert cert);
   */
 DLLEXPORT int tqsl_importTQSLFile(const char *file, int(*cb)(int type, const char *message, void *userdata), void *user);
 
+/** Get the serial for the first user cert from a .tq6 file
+  * used to support asking the user to save their cert after import
+  * \li \c file is the path to the file
+  * \li \c serial is where the serial number is returned
+  *
+  * Returns 0 on success, nonzero on failure.
+  *
+  */
+DLLEXPORT int tqsl_getSerialFromTQSLFile(const char *file, long *serial);
+
 /** Get the number of certificate providers known to tqsllib.
   */
 DLLEXPORT int tqsl_getNumProviders(int *n);
@@ -889,6 +909,16 @@ DLLEXPORT int tqsl_getLocationFieldInputType(tQSL_Location loc, int field_num, i
   */
 DLLEXPORT int tqsl_getLocationFieldDataType(tQSL_Location loc, int field_num, int *type);
 
+/** Get the flags for the input field.
+  *
+  * \c flags  will be either
+  * TQSL_LOCATION_FIELD_UPPER		Field is to be uppercased on input
+  * TQSL_LOCATION_FIELD_MUSTSEL		Value must be selected
+  * TQSL_LOCATION_FIELD_SELNXT		Value must be selected to allow Next/Finish
+  *
+  */
+DLLEXPORT int tqsl_getLocationFieldFlags(tQSL_Location loc, int field_num, int *flags);
+
 /** Get the length of the input field data. */
 DLLEXPORT int tqsl_getLocationFieldDataLength(tQSL_Location loc, int field_num, int *rval);
 
@@ -956,6 +986,10 @@ DLLEXPORT int tqsl_getDXCCEntity(int index, int *number, const char **name);
   */
 DLLEXPORT int tqsl_getDXCCEntityName(int number, const char **name);
 
+/** Get the zonemap  of a DXCC Entity by its DXCC number.
+  */
+DLLEXPORT int tqsl_getDXCCZoneMap(int number, const char **zonemap);
+
 /** Get the number of Band entries in the Band list */
 DLLEXPORT int tqsl_getNumBand(int *number);
 
